"""
TexNew

Automatic LaTeX template management.

:copyright: (c) 2019 by Alex Rutar
:license: MIT, see LICENSE for more details.
"""

__version__ = "1.0"

from texnew.test import parse_errors
from texnew.update import update
from texnew.core import run
from texnew.test import test
